% som_animate  animates the progress of a ring-SOM on 2 dimensional 
%    data points.
%   som_animate(DataFile,animate) plots the progress of a SOM
%   algorithm given the data in DataFile
%
%   Input:
%     DataFile - name of file to read data to
%     pause - time to pause for each iteration
%
% Because of the extensive redrawing required, the following may not 'flow'
% Use som_movie for better visulation. (which requires more memory,
% computation)
%
% Author: Dale Patterson
% $Version: 2.0.1 $ $Date: 3.30.06 $
% 
function som_animate(Report,wait)
% initialize defaults if any
if nargin < 2
    wait = 0.05;
end

% PLOT the basic figure and set up the display
t = 0:pi/20:2*pi;          % plot a yellow circle of
plot(sin(t),cos(t),'y');   % radius 1 @ origin [0,0]
axis([-1.1,1.1,-1.1,1.1]); % move the axis out from edge of the circle
axis square;               % and make x and y axis of same length
set(gca,'XTick',0:-1);     % set x-axis ticks off
set(gca,'YTick',0:-1);     % and y-saxis ticks off   
    
% open the file
fid = fopen(Report,'r');
    
% read the network paramaters and input points
net_params = fread(fid,[1,3]);
p = net_params(1);
n = net_params(2);
m = net_params(3);
input = fread(fid,[p,n],'double');

% colors is a matrix of colors one for each output node in the net, 
% nodes is a vector to store the handles for each node, points is 
% vector to store handles for each point and clusters is a cell array
% of points currently associated with each node
colors =  [1 0 0;                 % 1 red
           1 0 1;                 % 2 magenta
           0.451 0.2627 0.2627;   % 3 brown
           0 0 1;                 % 4 blue
           0 1 0;                 % 5 green
           0.8706 0.4902 0;       % 6 orange 
           0.5 0.5 0.5;           % 7 gray
           0.4784 0.06275 0.8941; % 8 purple
           0 1 1;                 % 9 cyan
           0 0 0;                 % 10 black
           0.6 0 0;               % 11 maroon
           0.4 0.4 0];            % 12 army green
nodes = zeros(m,1);   % handles for node plots
points = zeros(m,1);  % handles for input plots

% read the first period paramaters and initial weights
pparams = fread(fid,[1,5],'double');
w = fread(fid,[m,n],'double'); 

% find and color plot the point clusters according to initial weight
clusters = clustered(input,w);

hold all
    for i=1:m
        if ~isempty(clusters{i}) 
            points(i) = plot(clusters{i}(:,1),clusters{i}(:,2),'*', ...
                            'MarkerEdgeColor',colors(i,:), ...
                            'MarkerFaceColor',colors(i,:), ...
                            'MarkerSize',6);
            set(points(i),'EraseMode','xor','MarkerSize',6);
        else
            points(i) = NaN;
        end
    end
hold off   

% plot initial weights
% we have to plot the weights as two seperate graphs: lines and nodes
% because the lines connect a node to its adjacent nodes, the first and 
% last node are connected, but when plotting the node markers we only 
% want one node
lw = [w;w(1,:)];                    % connect w(1) to w(end)
hold all
    h = plot(lw(:,1),lw(:,2),'-k'); % plot lines seperate from
    set(h,'EraseMode','xor','MarkerSize',2);
    for i=1:m
        nodes(i) = plot(w(i,1),w(i,2),'o', ...
                        'MarkerEdgeColor',colors(i,:), ...
                        'MarkerFaceColor',colors(i,:));
        set(nodes(i),'EraseMode','xor','MarkerSize',8);
    end    
hold off

% set the title
title(['epoch: ',num2str(floor(pparams(1)/p)), ...
       '  \eta:',num2str(pparams(2)), ... 
       '  d:',num2str(pparams(3))]);

% and the legend
%legend('Unit Circle','Input','Adjacent','Node 1','Node 2','Node 3', ... 
%       'Node 4', 'Node 5','Node 6','Node 7','Node 8','Node 9','Node 10', ... 
%       'Node 11', 'Node 12','Location','EastOutside');
   
% loop over and plot the progress of the weights in the file
while ~feof(fid)
    pparams = fread(fid,[1,5],'double');
    if ~isempty(pparams)
        % read the next weight entry
        w = fread(fid,[m,n],'double'); 
        
        % find and color plot the clusters
        clusters = clustered(input,w);
        for i=1:m
            if ~isempty(clusters{i}) 
                if ~isnan(points(i))
                    set(points(i),'XData',clusters{i}(:,1), ...
                                  'YData',clusters{i}(:,2));
                 else
                    hold on
                    points(i) = plot(clusters{i}(:,1),clusters{i}(:,2),'*', ...
                            'MarkerEdgeColor',colors(i,:), ...
                            'MarkerFaceColor',colors(i,:), ...
                            'MarkerSize',6);
                    set(points(i),'EraseMode','xor','MarkerSize',6);
                    hold off
                end
            end
        end                 
        
        % plot the nodes
        lw = [w;w(1,:)];
        set(h,'XData',lw(:,1),'YData',lw(:,2));
        for i=1:m
            set(nodes(i),'XData',w(i,1),'YData',w(i,2));
        end 
                
        % set the title
        title(['epoch: ',num2str(floor(pparams(1)/p)), ...
               '  \eta:',num2str(pparams(2)), ... 
               '  d:',num2str(pparams(3))]);        
    
        drawnow;     
        pause(wait);
    end
end   
fclose(fid);
