% som_movie  saves the progress of som as a matlab movie
%   som_movie(DataFile,Mov,start,end) plots the progress of a SOM
%   algorithm given the data in DataFile and saves the movie
%   as a matlab movie
%
%   Input:
%     DataFile - name of file to read data from
%     MovFile - file to save the .mat extension file in. This file can be 
%     loaded and played using movie or saved to an avi file using:
%     start - time (iteration) to start recording
%     end - time (iteration) to stop recording
%     NOTE: due to the size of the files, the movie should not be more 
%     than 30 epochs or 1800 iterations
%     Play as matlab movie
%      load MovFile;
%      movie(Movie);
%     Save to avi
%      load MovFile;
%      movie2avi(Movie,Avi,'quality',100,'compression','None');
%
%
% Author: Dale Patterson
% $Version: 2.0.1 $ $Date: 3.30.06 $
% 
function som_movie(Data,Avi,start,stop)
if start > stop
    error('som_movie:invalidParameters','Start time must be less than stop time');
end

if stop - start > 2000
    error('som_movie:invalidParameters','Not enough memory to record specified periods');
end

% get data and keep only the specified time frame, and required paramaters
[net,input,pparams,weights] = load_som(Data);
pparams = pparams(:,1:3,start:stop); % only need t,nt,dt
weights = weights(:,:,start:stop);

p = net(1);
n = net(2);
m = net(3);
clear net;

% frame is the current frame
frame = 1;

% colors is a matrix of colors one for each output node in the net, 
% nodes is a vector to store the handles for each node, points is 
% vector to store handles for each point and clusters is a cell array
% of points currently associated with each node
colors =  [1 0 0;                 % 1 red
           1 0 1;                 % 2 magenta
           0.451 0.2627 0.2627;   % 3 brown
           0 0 1;                 % 4 blue
           0 1 0;                 % 5 green
           0.8706 0.4902 0;       % 6 orange 
           0.5 0.5 0.5;           % 7 gray
           0.4784 0.06275 0.8941; % 8 purple
           0 1 1;                 % 9 cyan
           0 0 0;                 % 10 black
           0.6 0 0;               % 11 maroon
           0.4 0.4 0];            % 12 army green
nodes = zeros(m,1);   % handles for node plots

% PLOT the basic figure and set up the display
t = 0:pi/20:2*pi;          % plot a yellow circle of
plot(sin(t),cos(t),'y');   % radius 1 @ origin [0,0]
axis([-1.1,1.1,-1.1,1.1]); % move the axis out from edge of the circle
axis square;               % and make x and y axis of same length
set(gca,'XTick',-1:0.5:1); % set x-axis 
set(gca,'YTick',-1:0.5:1); % and y-saxis ticks    
    
% read the first period paramaters and weights, removing them from the 
% respective multidimensional arrays
params = pparams(:,:,1);
pparams(:,:,1) = [];
w = weights(:,:,1);
weights(:,:,1) = [];

% find and color plot the point clusters according to the weight
clusters = clustered(input,w);

hold all
    for i=1:m
        if ~isempty(clusters{i}) 
            plot(clusters{i}(:,1),clusters{i}(:,2),'*', ...
                  'MarkerEdgeColor',colors(i,:), ...
                  'MarkerFaceColor',colors(i,:), ...
                  'MarkerSize',6);
        end
    end
hold off   

% plot first weights
% we have to plot the weights as two seperate graphs: lines and nodes
% because the lines connect a node to its adjacent nodes, the first and 
% last node are connected, but when plotting the node markers we only 
% want one node
lw = [w;w(1,:)];                    % connect w(1) to w(end)
hold all
    h = plot(lw(:,1),lw(:,2),'-k'); % plot lines seperate from
    for i=1:m
        nodes(i) = plot(w(i,1),w(i,2),'o', ...
                        'MarkerEdgeColor',colors(i,:), ...
                        'MarkerFaceColor',colors(i,:));
    end    
hold off

% set the title
txt = text(-1.08,1.05,['epoch: ',num2str(floor(pparams(1)/p)), ...
                      '  \eta:',num2str(pparams(2)), ... 
                      '  d:',num2str(pparams(3))]);
set(txt,'EraseMode','xor');      

% save the initial frame
Movie(frame) = getframe;
frame = frame + 1;
   
% loop over and plot the progress of the weights in the file
while ~isempty(weights)
    % get clusters and plot them
    clusters = clustered(input,weights(:,:,1));
    for i=1:m
        if ~isempty(clusters{i})
            hold all
                plot(clusters{i}(:,1),clusters{i}(:,2),'*', ...
                     'MarkerEdgeColor',colors(i,:), ...
                     'MarkerFaceColor',colors(i,:), ...
                     'MarkerSize',6);
            hold off                
        end
    end
    
    % plot the nodes
    lw = [weights(:,:,1);weights(1,:,1)];
    set(h,'XData',lw(:,1),'YData',lw(:,2));
    for i=1:m
        set(nodes(i),'XData',weights(i,1,1),'YData',weights(i,2,1));
    end 
    
    % set the title
%    title(['epoch: ',num2str(floor(pparams(:,1,1)/p)), ...
%           '  \eta:',num2str(pparams(:,2,1)), ... 
%           '  d:',num2str(pparams(:,3,1))]);        
    set(txt,'string',['epoch: ',num2str(floor(pparams(1)/p)), ...
                      '  \eta:',num2str(pparams(2)), ... 
                      '  d:',num2str(pparams(3))]);
    
    % capture the frame
    Movie(frame) = getframe;
            
    % set for next iteration
    pparams(:,:,1) = [];
    weights(:,:,1) = [];
    frame = frame + 1;
end           
        
% this uses a lot of memory
%save Mov Movie;
movie2avi(Movie,Avi,'quality',100);