% som_plot  plots a snapshot of som.
%   som_plot(DataFile,Time) plots a snapshot(s) of the weight vectors
%   for a som network at up to 4 time points.
%
%   Input:
%     DataFile - name of file to read data from
%     Time - a vector of 1 to 4 times to plot weights from, use -1 to specify
%     the final weight vectors. Time must be in ascending order.
%
%
% Author: Dale Patterson
% $Version: 1.0.3 $ $Date: 4.03.06 $
% 
function som_plot(Data,time)
% verify time
plots = length(time);
if plots <= 0 || plots > 4 
    error('plot_som:timeDimensionInvalid','Time: must be of size 1-4');
end

% open the data file
[fid,fmsg] = fopen(Data,'r');
if fid == -1
    error('plot_som:cannotOpenDataFile',fmsg);
end

% read in network paramaters and input
net_params = fread(fid,[1,3]);
p = net_params(1);
n = net_params(2);
m = net_params(3);
input = fread(fid,[p,n],'double');
% fid is currently at initial weights

% read period paramaters and weights from file
% som halts at 120 epochs, an epoch is of size p and we need one additional
% for the initial weights and period paramaters
params = ones(1,3,120*p+1) .* -1;
weights = zeros(m,n,120*p+1);

% read the data
page = 1;
while ~feof(fid)
    pparams = fread(fid,[1,5],'double');
    if ~isempty(pparams)
        params(:,:,page) = pparams(1:3); % dont need distance & winner
        weights(:,:,page) = fread(fid,[m,n],'double');    
        page = page + 1;
    end
end

% close the file
fclose(fid);

% remove unused pages
k = find(params(1,1,:) ~= -1); % k = index of all valid pages
params = params(:,:,k);        
weights = weights(:,:,k);

% now plot the specified snapshots
if plots == 1
    if time(1) == -1
        plot_som2(input,p,n,m,params(:,:,end),weights(:,:,end));
    else
        t = time(1) + 1;
        plot_som2(input,p,n,m,params(:,:,t),weights(:,:,t));
    end
elseif plots == 2
    subplot(1,2,1);
    t = time(1) + 1;
    plot_som2(input,p,n,m,params(:,:,t),weights(:,:,t));
    subplot(1,2,2);
    if time(2) == -1
        plot_som2(input,p,n,m,params(:,:,end),weights(:,:,end));
    else
        t = time(2) + 1;
        plot_som2(input,p,n,m,params(:,:,t),weights(:,:,t));
    end    
elseif plots == 3
    subplot(2,2,1);
    t = time(1) + 1;
    plot_som2(input,p,n,m,params(:,:,t),weights(:,:,t));
    subplot(2,2,2);
    t = time(2) + 1;
    plot_som2(input,p,n,m,params(:,:,t),weights(:,:,t));
    subplot(2,2,3);
    if time(3) == -1
        plot_som2(input,p,n,m,params(:,:,end),weights(:,:,end));
    else
        t = time(3) + 1;
        plot_som2(input,p,n,m,params(:,:,t),weights(:,:,t));
    end    
elseif plots == 4
    subplot(2,2,1);
    t = time(1) + 1;
    plot_som2(input,p,n,m,params(:,:,t),weights(:,:,t));
    subplot(2,2,2);
    t = time(2) + 1;
    plot_som2(input,p,n,m,params(:,:,t),weights(:,:,t));
    subplot(2,2,3);
    t = time(3) + 1;
    plot_som2(input,p,n,m,params(:,:,t),weights(:,:,t));    
    subplot(2,2,4);
    if time(4) == -1
        plot_som2(input,p,n,m,params(:,:,end),weights(:,:,end));
    else
        t = time(4) + 1;
        plot_som2(input,p,n,m,params(:,:,t),weights(:,:,t));
    end        
end

% plot_som2(input,p,n,m,params,weights)
%  plot the som snapshot
function plot_som2(input,p,n,m,params,weights)
t = 0:pi/20:2*pi;          % plot a yellow circle of
plot(sin(t),cos(t),'y');   % radius 1 @ origin [0,0]
axis([-1.1,1.1,-1.1,1.1]); % move the axis out from edge of the circle
axis square;               % and make x and y axis of same length
set(gca,'XTick',0:-1);     % set x-axis ticks off
set(gca,'YTick',0:-1);     % and y-axis ticks off   

% colors is a list of colors for each node and associated points, clusters
% holds the points assocatiated with each node
colors =  [1 0 0;                 % 1 red
           1 0 1;                 % 2 magenta
           0.451 0.2627 0.2627;   % 3 brown
           0 0 1;                 % 4 blue
           0 1 0;                 % 5 green
           0.8706 0.4902 0;       % 6 orange 
           0.5 0.5 0.5;           % 7 gray
           0.4784 0.06275 0.8941; % 8 purple
           0 1 1;                 % 9 cyan
           0 0 0;                 % 10 black
           0.6 0 0;               % 11 maroon
           0.4 0.4 0];            % 12 army green

% find and color plot the point clusters according to initial weight
clusters = clustered(input,weights);

hold all
    for i=1:m
        if ~isempty(clusters{i}) 
            points(i) = plot(clusters{i}(:,1),clusters{i}(:,2),'*', ...
                            'MarkerEdgeColor',colors(i,:), ...
                            'MarkerFaceColor',colors(i,:), ...
                            'MarkerSize',6);
        end
    end
hold off   

% plot weights and adjacent lines
lw = [weights;weights(1,:)]; % connect weights(1) to weights(end)
hold all
    plot(lw(:,1),lw(:,2),'-k'); % plot lines seperate from
    for i=1:m
        plot(weights(i,1),weights(i,2),'o', ... 
             'MarkerEdgeColor',colors(i,:), ...
             'MarkerFaceColor',colors(i,:));
    end    
hold off

% set the title
title(['epoch: ',num2str(floor(params(1)/p)), ...
       '  \eta:',num2str(params(2)), ... 
       '  d:',num2str(params(3))]);